#include "StdAfx.h"
#include "Calendar.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

char weekdays[8][4] =
{
	{"Lun"}, {"Mar"}, {"Mer"}, {"Jeu"}, {"Ven"}, {"Sam"}, {"Dim"}
};
char monthsName[12][10] =
{
	{"Janvier"}, {"Fvrier"}, {"Mars"}, {"Avril"}, {"Mai"}, {"Juin"}, {"Juillet"}, {"Aot"}, {"Septembre"}, {"Octobre"}, {"Novembre"}, {"Dcembre"}
};
char days[42][4];
char month[] =
{
	31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
};

C_Calendar::C_Calendar()
{
	// Default Values //
	m_nWidth = 0;
	m_nHeight = 0;
	m_hMemDC = NULL;
	m_hBmp = NULL;

	m_szFontName = _T("Garamond");
	m_crTextColor = RGB(0, 255, 0);			// pure green
	m_crBackColor = RGB(255, 255, 255);		// pure white
	m_nTextSize = 24;
}

C_Calendar::~C_Calendar()
{
	if (NULL != m_hBmp)
	{
		::DeleteObject(m_hBmp);
		m_hBmp = NULL;
	}
	if (NULL != m_hMemDC)
	{
		::DeleteDC(m_hMemDC);
		m_hMemDC = NULL;
	}
}

VOID C_Calendar::SetTextSize(INT nTextSize)
{
	if (nTextSize > 0)
		m_nTextSize = nTextSize;
}

VOID C_Calendar::SetFont(CString szFont)
{
	if (!szFont.IsEmpty())
		m_szFontName = szFont;
}

VOID C_Calendar::SetTextColor(COLORREF crTextColor)
{
	m_crTextColor = crTextColor;
}

VOID C_Calendar::SetBackColor(COLORREF crBackColor)
{
	m_crBackColor = crBackColor;
}

HDC C_Calendar::DrawCalendar(HDC hDC)
{
	// Finding the text width and height of the font to be used //
	TEXTMETRIC tmText;
	HFONT hFont, hOldFont;
	HPEN hPen, hOldPen;
	RECT CalRect;
	RECT RectDay;
	HBRUSH hBackBrush = CreateSolidBrush(m_crBackColor);
	INT nXPos, nYPos;

	if (NULL != m_hBmp)
	{
		::DeleteObject(m_hBmp);
		m_hBmp = NULL;
	}
	if (NULL != m_hMemDC)
	{
		::DeleteDC(m_hMemDC);
		m_hMemDC = NULL;
	}
	m_hMemDC = ::CreateCompatibleDC(hDC);
	m_hBmp = ::CreateCompatibleBitmap(hDC, 800, 600);
	::SelectObject(m_hMemDC, m_hBmp);

	hFont = CreateFont(m_nTextSize, 0, 0, 0, 0, 0, 0, 0, DEFAULT_CHARSET, 0, 0, 0, FF_DONTCARE, m_szFontName); 
	hPen = ::CreatePen(PS_SOLID, 1, m_crTextColor);

	hOldFont = (HFONT)::SelectObject(m_hMemDC, hFont);
	hOldPen = (HPEN)::SelectObject(m_hMemDC, hPen);
	::GetTextMetrics(m_hMemDC, &tmText);
	::SetTextColor(m_hMemDC, m_crTextColor);
	::SetBkMode(m_hMemDC, TRANSPARENT);

	m_nCharWidth = (tmText.tmMaxCharWidth + 2 * tmText.tmAveCharWidth);	
	m_nCharHeight = tmText.tmHeight + 4;							

	m_nWidth = (CAL_COLS)*m_nCharWidth; 
	m_nWidth += m_nCharWidth / 2;
	m_nHeight = (CAL_ROWS + 1) * m_nCharHeight;	
	m_nWidth += (m_nWidth % 2);
	m_nHeight += (m_nHeight % 2);

	// fill back ground color 
	CalRect.left = CalRect.top = 0;
	CalRect.right = m_nWidth;
	CalRect.bottom = m_nHeight;
	::FillRect(m_hMemDC, &CalRect, hBackBrush);

	CTime t = CTime::GetCurrentTime();
	CTime dt(t.GetYear(), t.GetMonth(), 1, 0, 0, 0);
	INT nDay = dt.GetDayOfWeek() - 2;
	INT nMonth = t.GetMonth();
	INT nYear = t.GetYear();

	if (nDay < 0)
		nDay += 7;
	// Check for leap year 
	if ((nYear % 400) == 0 || ((nYear % 4) == 0 && (nYear % 100) != 0))
		month[1] = 29;

	CHAR day[4];

	memset(days, '\0', 42 * 4);

	CString tmp;
	tmp.Format("%s %d", monthsName[nMonth - 1], nYear);

	RectDay.left = 0;
	RectDay.top = 0;
	RectDay.right = tmp.GetLength() * tmText.tmMaxCharWidth / 2;
	RectDay.bottom = m_nCharHeight;

	DrawText(m_hMemDC, tmp, tmp.GetLength(), &RectDay, DT_CENTER | DT_VCENTER | DT_SINGLELINE);				

	for (INT i = (nDay),j = 1; i < 42 && j <= month[nMonth - 1] ; i++)
	{
		itoa(j, day, 10);
		strcpy(days[i], day);
		j++;
		if (i == 41)
			i = -1;
	}

	month[1] = 28;

	nXPos = nYPos = 0;
	//nYPos -=(m_nCharHeight/2);

	for (int n = 0; n < (CAL_ROWS* CAL_COLS); n++)
	{
		if (!(n % 7))
		{
			nXPos = 0;
			nYPos += m_nCharHeight;
		}

		RectDay.left = nXPos;
		RectDay.top = nYPos;
		RectDay.right = RectDay.left + m_nCharWidth;
		RectDay.bottom = RectDay.top + m_nCharHeight;

		if (n < 7)
		{
			DrawText(m_hMemDC, weekdays[n], (int)strlen(weekdays[n]), &RectDay, DT_CENTER | DT_VCENTER | DT_SINGLELINE);
		}
		else
		{
			if (strlen(days[n - 7]) != 0)
			{
				if (t.GetDay() == atoi(days[n - 7]))
				{
					::FillRect(m_hMemDC, &RectDay, CreateSolidBrush(GetSysColor(COLOR_DESKTOP)));
					DrawText(m_hMemDC, days[n - 7], strlen(days[n - 7]), &RectDay, DT_CENTER | DT_VCENTER | DT_SINGLELINE);
				}
				else
					DrawText(m_hMemDC, days[n - 7], strlen(days[n - 7]), &RectDay, DT_CENTER | DT_VCENTER | DT_SINGLELINE);
			}
		}
		nXPos += m_nCharWidth;
	}

	::SelectObject(m_hMemDC, hOldPen);
	::SelectObject(m_hMemDC, hOldFont);
	::DeleteObject(hPen);
	::DeleteObject(hBackBrush);
	::DeleteObject(hFont);

	return (m_hMemDC);
}


int C_Calendar::GetHeight()
{
	return (m_nHeight);
}

int C_Calendar::GetWidth()
{
	return (m_nWidth);
}